@@ -54,7 +54,7 @@ wnck_workspace_accessible_get_type (void)
 
   if (!type)
     {
-      static GTypeInfo tinfo = 
+      const GTypeInfo tinfo = 
       {
         sizeof (WnckWorkspaceAccessibleClass),
         (GBaseInitFunc) NULL, /* base init */
@@ -68,7 +68,7 @@ wnck_workspace_accessible_get_type (void)
         NULL /* value table */
       };
 
-      static const GInterfaceInfo atk_component_info = 
+      const GInterfaceInfo atk_component_info = 
       {
         (GInterfaceInitFunc) atk_component_interface_init,
         (GInterfaceFinalizeFunc) NULL,
@@ -150,10 +150,10 @@ wnck_workspace_accessible_get_size (AtkComponent *component,
                                     int          *width,
                                     int          *height)
 {
-  AtkCoordType coords; 
+  AtkCoordType coords = ATK_XY_SCREEN; 
   int x, y;
 
-  /* FIXME this is blatantly broken (coords uninitialized) */
+  /* FIXME: Value for initialization of coords picked randomly to please gcc */
   
   wnck_workspace_accessible_get_extents (component, &x, &y, width, height, coords);
 }
