@@ -245,11 +245,13 @@ text_property_to_utf8 (const XTextProperty *prop)
                                           &list);
 
   if (count == 0)
-    return NULL;
+    retval = NULL;
+  else
+    {
+      retval = list[0];
+      list[0] = g_strdup (""); /* something to free */
+    }
 
-  retval = list[0];
-  list[0] = g_strdup (""); /* something to free */
-  
   g_strfreev (list);
 
   return retval;
@@ -293,7 +295,7 @@ _wnck_get_string_property_latin1 (Window  xwindow,
   int format;
   gulong nitems;
   gulong bytes_after;
-  guchar *str;
+  gchar *str;
   int err, result;
   char *retval;
   
@@ -331,7 +333,7 @@ _wnck_get_utf8_property (Window  xwindow,
   int format;
   gulong nitems;
   gulong bytes_after;
-  guchar *val;
+  gchar *val;
   int err, result;
   char *retval;
   Atom utf8_string;
@@ -625,7 +627,7 @@ _wnck_set_utf8_list (Window   xwindow,
 		   xwindow,
                    atom,
 		   utf8_string, 8, PropModeReplace,
-		   flattened->str, flattened->len);
+		   (guchar *) flattened->str, flattened->len);
   
   _wnck_error_trap_pop ();
 
@@ -2214,7 +2216,8 @@ _wnck_get_window_geometry (Screen *screen,
                            int    *widthp,
                            int    *heightp)
 {
-  int x, y, width, height, bw, depth;
+  int x, y;
+  unsigned int width, height, bw, depth;
   Window root_window;
 
   width = 1;
@@ -2237,6 +2240,36 @@ _wnck_get_window_geometry (Screen *screen,
     *heightp = height;
 }
 
+void _wnck_set_window_geometry (Screen *screen,
+                                Window  xwindow,
+                                int     gravity_and_flags,
+                                int     x,
+                                int     y,
+                                int     width,
+                                int     height)
+{
+  XEvent xev;
+
+  xev.xclient.type = ClientMessage;
+  xev.xclient.serial = 0;
+  xev.xclient.send_event = True;
+  xev.xclient.display = gdk_display;
+  xev.xclient.window = xwindow;
+  xev.xclient.message_type = _wnck_atom_get ("_NET_MOVERESIZE_WINDOW");
+  xev.xclient.format = 32;
+  xev.xclient.data.l[0] = gravity_and_flags;
+  xev.xclient.data.l[1] = x;
+  xev.xclient.data.l[2] = y;
+  xev.xclient.data.l[3] = width;
+  xev.xclient.data.l[4] = height;
+
+  XSendEvent (gdk_display,
+              RootWindowOfScreen (screen),
+              False,
+              SubstructureRedirectMask | SubstructureNotifyMask,
+              &xev);
+}
+
 void
 _wnck_get_window_position (Screen *screen,
 			   Window  xwindow,
